import logging
import subprocess
import glob
import pkgutil
import hashlib
import importlib
from karton.core import Config, Karton, Task, Resource
from typing import Optional

from .__version__ import __version__

log = logging.getLogger(__name__)

def unpacker_module_worker(sample, module) -> Task:
    spec = importlib.util.spec_from_file_location("module.name", module)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    module = module.KartonUnpackerModule(sample=sample)
    if module.enabled is True:
        return module.main()

class Unpacker(Karton):

    """
    A modular Karton Framework service that unpacks common packers like UPX, MPress and others using the Qilling Framework.
    """

    identity = "karton.unpacker"

    filters = [
        {"type": "sample", "stage": "recognized", "kind": "runnable", "platform": "win32"},
        {"type": "sample", "stage": "recognized", "kind": "runnable", "platform": "win64" },
        {"type": "sample", "stage": "recognized", "kind": "runnable", "platform": "linux"}
    ]

    @classmethod
    def args_parser(cls):
        parser = super().args_parser()
        parser.add_argument("--modules", help="Modules Directory", required=True)
        return parser

    @classmethod
    def main(cls):
        parser = cls.args_parser()
        args = parser.parse_args()
        config = Config(args.config_file)
        service = Unpacker(config=config, modules=args.modules)
        service.loop()

    def __init__(self, modules: Optional[str] = None, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        modules = modules or "modules"
        self.modules = glob.glob(f'{modules}/**/*.py', recursive=True)

    def process(self, task: Task) -> None:
        sample = task.get_resource("sample")
        for module in self.modules:
            task = unpacker_module_worker(sample, module)
            if task is not None:
                log.info(f"successfully unpacked {sample.name}, sending task")
                self.send_task(task)

if __name__ == "__main__":
    Unpacker().loop()



#!/usr/bin/env python

import os
from glob import glob
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from pathlib import Path

version_path = Path(__file__).parent / "karton/unpacker/__version__.py"
version_info = {}
exec(version_path.read_text(), version_info)

def get_requirements(default):
    result = []
    requirement_files = [y for x in os.walk('modules') for y in glob(os.path.join(x[0], 'requirements.txt'))]
    requirement_files.append(default)
    for requirement_file in requirement_files:
        f = open(requirement_file, "r")
        result.extend(f.read().splitlines())
    return list(dict.fromkeys(result))

setup(
    name="karton-unpacker",
    version=version_info["__version__"],
    description="A modular Karton Framework service that unpacks common packers like UPX, MPress and others using the Qilling Framework.",
    namespace_packages=["karton"],
    packages=["karton.unpacker"],
    install_requires=get_requirements('requirements.txt'),
    entry_points={
        'console_scripts': [
            'karton-unpacker=karton.unpacker:Unpacker.main'
        ],
    },
    classifiers=[
        "Programming Language :: Python",
        "Operating System :: OS Independent",
    ],
)
